@@ -38,6 +38,7 @@ from mitmproxy import flow
 from mitmproxy import platform
 from mitmproxy.connection import Address
 from mitmproxy.net import local_ip
+from mitmproxy.net.free_port import get_free_port
 from mitmproxy.proxy import commands
 from mitmproxy.proxy import layers
 from mitmproxy.proxy import mode_specs
@@ -184,8 +185,11 @@ class ServerInstance(Generic[M], metaclass=ABCMeta):
     async def handle_stream(
         self,
         reader: asyncio.StreamReader | mitmproxy_rs.Stream,
-        writer: asyncio.StreamWriter | mitmproxy_rs.Stream,
+        writer: asyncio.StreamWriter | mitmproxy_rs.Stream | None = None,
     ) -> None:
+        if writer is None:
+            assert isinstance(reader, mitmproxy_rs.Stream)
+            writer = reader
         handler = ProxyConnectionHandler(
             ctx.master, reader, writer, ctx.options, self.mode
         )
@@ -204,7 +208,8 @@ class ServerInstance(Generic[M], metaclass=ABCMeta):
                 handler.layer.context.client.sockname = original_dst
                 handler.layer.context.server.address = original_dst
         elif isinstance(
-            self.mode, (mode_specs.WireGuardMode, mode_specs.LocalMode)
+            self.mode,
+            (mode_specs.WireGuardMode, mode_specs.LocalMode, mode_specs.TunMode),
         ):  # pragma: no cover on platforms without wg-test-client
             handler.layer.context.server.address = writer.get_extra_info(
                 "remote_endpoint", handler.layer.context.client.sockname
@@ -213,12 +218,13 @@ class ServerInstance(Generic[M], metaclass=ABCMeta):
         with self.manager.register_connection(handler.layer.context.client.id, handler):
             await handler.handle_client()
 
-    async def handle_udp_stream(self, stream: mitmproxy_rs.Stream) -> None:
-        await self.handle_stream(stream, stream)
-
 
 class AsyncioServerInstance(ServerInstance[M], metaclass=ABCMeta):
-    _servers: list[asyncio.Server | mitmproxy_rs.udp.UdpServer]
+    _servers: list[
+        asyncio.Server
+        | mitmproxy_rs.udp.UdpServer
+        | mitmproxy_rs.wireguard.WireGuardServer
+    ]
 
     def __init__(self, *args, **kwargs) -> None:
         self._servers = []
@@ -232,7 +238,9 @@ class AsyncioServerInstance(ServerInstance[M], metaclass=ABCMeta):
     def listen_addrs(self) -> tuple[Address, ...]:
         addrs = []
         for s in self._servers:
-            if isinstance(s, mitmproxy_rs.udp.UdpServer):
+            if isinstance(
+                s, (mitmproxy_rs.udp.UdpServer, mitmproxy_rs.wireguard.WireGuardServer)
+            ):
                 addrs.append(s.getsockname())
             else:
                 try:
@@ -270,96 +278,72 @@ class AsyncioServerInstance(ServerInstance[M], metaclass=ABCMeta):
 
     async def listen(
         self, host: str, port: int
-    ) -> list[asyncio.Server | mitmproxy_rs.udp.UdpServer]:
+    ) -> list[
+        asyncio.Server
+        | mitmproxy_rs.udp.UdpServer
+        | mitmproxy_rs.wireguard.WireGuardServer
+    ]:
         if self.mode.transport_protocol not in ("tcp", "udp", "both"):
             raise AssertionError(self.mode.transport_protocol)
 
-        servers: list[asyncio.Server | mitmproxy_rs.udp.UdpServer] = []
-        if self.mode.transport_protocol in ("tcp", "both"):
-            # workaround for https://github.com/python/cpython/issues/89856:
-            # We want both IPv4 and IPv6 sockets to bind to the same port.
-            # This may fail (https://github.com/mitmproxy/mitmproxy/pull/5542#issuecomment-1222803291),
-            # so we try to cover the 99% case and then give up and fall back to what asyncio does.
-            if port == 0:
-                try:
-                    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                    s.bind(("", 0))
-                    port = s.getsockname()[1]
-                    s.close()
-                    servers.append(
-                        await asyncio.start_server(self.handle_stream, host, port)
-                    )
-                except Exception as e:
-                    logger.debug(
-                        f"Failed to listen on a single port ({e!r}), falling back to default behavior."
-                    )
-                    port = 0
-                    servers.append(
-                        await asyncio.start_server(self.handle_stream, host, port)
-                    )
-            else:
-                servers.append(
-                    await asyncio.start_server(self.handle_stream, host, port)
+        # workaround for https://github.com/python/cpython/issues/89856:
+        # We want both IPv4 and IPv6 sockets to bind to the same port.
+        # This may fail (https://github.com/mitmproxy/mitmproxy/pull/5542#issuecomment-1222803291),
+        # so we try to cover the 99% case and then give up and fall back to what asyncio does.
+        if port == 0:
+            try:
+                return await self.listen(host, get_free_port())
+            except Exception as e:
+                logger.debug(
+                    f"Failed to listen on a single port ({e!r}), falling back to default behavior."
                 )
+
+        servers: list[
+            asyncio.Server
+            | mitmproxy_rs.udp.UdpServer
+            | mitmproxy_rs.wireguard.WireGuardServer
+        ] = []
+        if self.mode.transport_protocol in ("tcp", "both"):
+            servers.append(await asyncio.start_server(self.handle_stream, host, port))
         if self.mode.transport_protocol in ("udp", "both"):
             # we start two servers for dual-stack support.
             # On Linux, this would also be achievable by toggling IPV6_V6ONLY off, but this here works cross-platform.
             if host == "":
-                ipv4 = await mitmproxy_rs.udp.start_udp_server(
-                    "0.0.0.0",
-                    port,
-                    self.handle_udp_stream,
-                )
+                ipv4 = await self.start_udp_based_server("0.0.0.0", port)
                 servers.append(ipv4)
                 try:
-                    ipv6 = await mitmproxy_rs.udp.start_udp_server(
-                        "[::]",
-                        ipv4.getsockname()[1],
-                        self.handle_udp_stream,
+                    ipv6 = await self.start_udp_based_server(
+                        "::", ipv4.getsockname()[1]
                     )
                     servers.append(ipv6)  # pragma: no cover
                 except Exception:  # pragma: no cover
                     logger.debug("Failed to listen on '::', listening on IPv4 only.")
             else:
-                servers.append(
-                    await mitmproxy_rs.udp.start_udp_server(
-                        host,
-                        port,
-                        self.handle_udp_stream,
-                    )
-                )
+                servers.append(await self.start_udp_based_server(host, port))
 
         return servers
 
+    async def start_udp_based_server(
+        self, host, port
+    ) -> mitmproxy_rs.udp.UdpServer | mitmproxy_rs.wireguard.WireGuardServer:
+        return await mitmproxy_rs.udp.start_udp_server(
+            host,
+            port,
+            self.handle_stream,
+        )
 
-class WireGuardServerInstance(ServerInstance[mode_specs.WireGuardMode]):
-    _server: mitmproxy_rs.wireguard.WireGuardServer | None = None
 
+class WireGuardServerInstance(AsyncioServerInstance[mode_specs.WireGuardMode]):
     server_key: str
     client_key: str
+    pubkey: str
 
     def make_top_layer(
         self, context: Context
     ) -> Layer:  # pragma: no cover on platforms without wg-test-client
         return layers.modes.TransparentProxy(context)
 
-    @property
-    def is_running(self) -> bool:
-        return self._server is not None
-
-    @property
-    def listen_addrs(self) -> tuple[Address, ...]:
-        if self._server:
-            return (self._server.getsockname(),)
-        else:
-            return tuple()
-
     async def _start(self) -> None:
-        assert self._server is None
-        host = self.mode.listen_host(ctx.options.listen_host)
-        port = self.mode.listen_port(ctx.options.listen_port)
-        assert port is not None
-
         if self.mode.data:
             conf_path = Path(self.mode.data).expanduser()
         else:
@@ -383,25 +367,31 @@ class WireGuardServerInstance(ServerInstance[mode_specs.WireGuardMode]):
             self.client_key = c["client_key"]
         except Exception as e:
             raise ValueError(f"Invalid configuration file ({conf_path}): {e}") from e
+
         # error early on invalid keys
-        p = mitmproxy_rs.wireguard.pubkey(self.client_key)
+        self.pubkey = mitmproxy_rs.wireguard.pubkey(self.client_key)
         _ = mitmproxy_rs.wireguard.pubkey(self.server_key)
 
-        self._server = await mitmproxy_rs.wireguard.start_wireguard_server(
-            host or "0.0.0.0",
-            port,
-            self.server_key,
-            [p],
-            self.wg_handle_stream,
-            self.wg_handle_stream,
-        )
+        await super()._start()
 
         conf = self.client_conf()
         assert conf
         logger.info("-" * 60 + "\n" + conf + "\n" + "-" * 60)
 
+    async def start_udp_based_server(
+        self, host, port
+    ) -> mitmproxy_rs.wireguard.WireGuardServer:
+        return await mitmproxy_rs.wireguard.start_wireguard_server(
+            host,
+            port,
+            self.server_key,
+            [self.pubkey],
+            self.handle_stream,
+            self.handle_stream,
+        )
+
     def client_conf(self) -> str | None:
-        if not self._server:
+        if not self._servers:
             return None
         host = (
             self.mode.listen_host(ctx.options.listen_host)
@@ -426,19 +416,6 @@ class WireGuardServerInstance(ServerInstance[mode_specs.WireGuardMode]):
     def to_json(self) -> dict:
         return {"wireguard_conf": self.client_conf(), **super().to_json()}
 
-    async def _stop(self) -> None:
-        assert self._server is not None
-        try:
-            self._server.close()
-            await self._server.wait_closed()
-        finally:
-            self._server = None
-
-    async def wg_handle_stream(
-        self, stream: mitmproxy_rs.Stream
-    ) -> None:  # pragma: no cover on platforms without wg-test-client
-        await self.handle_stream(stream, stream)
-
 
 class LocalRedirectorInstance(ServerInstance[mode_specs.LocalMode]):
     _server: ClassVar[mitmproxy_rs.local.LocalRedirector | None] = None
@@ -460,7 +437,7 @@ class LocalRedirectorInstance(ServerInstance[mode_specs.LocalMode]):
         stream: mitmproxy_rs.Stream,
     ) -> None:
         if cls._instance is not None:
-            await cls._instance.handle_stream(stream, stream)
+            await cls._instance.handle_stream(stream)
 
     async def _start(self) -> None:
         if self._instance:
@@ -523,6 +500,47 @@ class DnsInstance(AsyncioServerInstance[mode_specs.DnsMode]):
         return layers.DNSLayer(context)
 
 
+class TunInstance(ServerInstance[mode_specs.TunMode]):
+    _server: mitmproxy_rs.tun.TunInterface | None = None
+    listen_addrs = ()
+
+    def make_top_layer(
+        self, context: Context
+    ) -> Layer:  # pragma: no cover mocked in tests
+        return layers.modes.TransparentProxy(context)
+
+    @property
+    def is_running(self) -> bool:
+        return self._server is not None
+
+    @property
+    def tun_name(self) -> str | None:
+        if self._server:
+            return self._server.tun_name()
+        else:
+            return None
+
+    def to_json(self) -> dict:
+        return {"tun_name": self.tun_name, **super().to_json()}
+
+    async def _start(self) -> None:
+        assert self._server is None
+        self._server = await mitmproxy_rs.tun.create_tun_interface(
+            self.handle_stream,
+            self.handle_stream,
+            tun_name=self.mode.data or None,
+        )
+        logger.info(f"TUN interface created: {self._server.tun_name()}")
+
+    async def _stop(self) -> None:
+        assert self._server is not None
+        try:
+            self._server.close()
+            await self._server.wait_closed()
+        finally:
+            self._server = None
+
+
 # class Http3Instance(AsyncioServerInstance[mode_specs.Http3Mode]):
 #     def make_top_layer(self, context: Context) -> Layer:
 #         return layers.modes.HttpProxy(context)
